/* MamaP.h - private header for MandelSpawn master widget */
/* Copyright (C) 1990 Andreas Gustafsson */

#ifndef _MamaP_h
#define _MamaP_h

#include "Ms.h" /* public header of child widget */
#include "Mama.h"

#include "MsIPC.h"

typedef struct slave
{ char *name_string;		/* machine name of the slave */
  struct sockaddr_in name; 	/* network address of the slave */
  int has_timeout; 		/* the timeout has been initialized */
  XtIntervalId timer; 		/* timeout id (for removing the timeout) */
  unsigned int n_timeouts; 	/* how many times the slave has timed out */
  unsigned int n_packets;	/* number of packets that have arrived */
  unsigned int n_late_packets;	/* number of packets that arrived too late */
  unsigned long mi_count; 	/* number of iterations done by this slave */
  unsigned int no;		/* slave serial number */
  MamaWidget mama;		/* back pointer to the mama */
  int disabled;			/* slave disabled due to error */
  unsigned timeout;		/* timeout in milliseconds */
} slave;

typedef struct chunk
{ struct chunk *next;
  struct chunk *prev;
  int drawn;			/* true if at least one reply has arrived */
  MsWidget wid; 		/* pointer to widget owning this chunk */
  unsigned int no; 		/* serial number within current sequence */
  caddr_t client_data;		/* client data (unknown size) */
  caddr_t slave_data;		/* slave data */
  unsigned int slave_datalen;	/* length of slave data */
} chunk;

/* widget instance structure */
typedef struct 
{ unsigned n_colours;		/* number of colours */
  char *spectrum;		/* text definitions of our colours */
  unsigned n_slaves; 		/* number of slaves */
  struct slave **slaves;	/* array of pointers to slave descriptors */
  struct chunk to_draw; 	/* head of queue of chunks to be drawn */
  struct chunk drawn;		/* head of queue of chunks already drawn */
  int my_socket;  		/* socket for communicating with slaves */
  XtInputId input_id;		/* unique channel id */
  int sequence;			/* current sequence number */
  int n_chunks;			/* number of chunks in the active sequence */
  unsigned int max_chunks; 	/* current size of chunk index */
  struct chunk **chunks; 	/* pointer to the chunk index */
  int pid;			/* pid of this process */
  unsigned long *pixels;	/* pixel values for our colours */
  unsigned n_hues;		/* max. iterations */
  unsigned n_popups_created;	/* number of popups created (not existing) */
  Bool bw;			/* force black-and-white operation */
  Colormap my_colormap;		/* may be the default colormap */
  Bool wrap;			/* wrap colourmap when too few colours */
  Bool debug;			/* debugging mode */
} MamaPart;

typedef struct _MamaRec 
{ CorePart	core;
  MamaPart	mama;
} MamaRec;

/* Widget class structure */
typedef struct 
{ int dummy;
} MamaClassPart;

typedef struct _MsClassRec 
{ CoreClassPart core_class;
  MamaClassPart mama;
} MamaClassRec;

extern MamaClassRec mamaClassRec;
#endif /* _MamaP_h */
